"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RULE_NAME = void 0;
const node_utils_1 = require("../node-utils");
const create_testing_library_rule_1 = require("../create-testing-library-rule");
exports.RULE_NAME = 'await-async-utils';
exports.default = create_testing_library_rule_1.createTestingLibraryRule({
    name: exports.RULE_NAME,
    meta: {
        type: 'problem',
        docs: {
            description: 'Enforce promises from async utils to be handled',
            category: 'Best Practices',
            recommended: 'warn',
        },
        messages: {
            awaitAsyncUtil: 'Promise returned from `{{ name }}` must be handled',
            asyncUtilWrapper: 'Promise returned from {{ name }} wrapper over async util must be handled',
        },
        fixable: null,
        schema: [],
    },
    defaultOptions: [],
    create(context, _, helpers) {
        const functionWrappersNames = [];
        function detectAsyncUtilWrapper(node) {
            const innerFunction = node_utils_1.getInnermostReturningFunction(context, node);
            if (innerFunction) {
                functionWrappersNames.push(node_utils_1.getFunctionName(innerFunction));
            }
        }
        return {
            'CallExpression Identifier'(node) {
                if (helpers.isAsyncUtil(node)) {
                    detectAsyncUtilWrapper(node);
                    const closestCallExpression = node_utils_1.findClosestCallExpressionNode(node, true);
                    if (!closestCallExpression) {
                        return;
                    }
                    const references = node_utils_1.getVariableReferences(context, closestCallExpression.parent);
                    if (references && references.length === 0) {
                        if (!node_utils_1.isPromiseHandled(node)) {
                            return context.report({
                                node,
                                messageId: 'awaitAsyncUtil',
                                data: {
                                    name: node.name,
                                },
                            });
                        }
                    }
                    else {
                        for (const reference of references) {
                            const referenceNode = reference.identifier;
                            if (!node_utils_1.isPromiseHandled(referenceNode)) {
                                return context.report({
                                    node,
                                    messageId: 'awaitAsyncUtil',
                                    data: {
                                        name: referenceNode.name,
                                    },
                                });
                            }
                        }
                    }
                }
                else if (functionWrappersNames.includes(node.name)) {
                    if (!node_utils_1.isPromiseHandled(node)) {
                        return context.report({
                            node,
                            messageId: 'asyncUtilWrapper',
                            data: { name: node.name },
                        });
                    }
                }
            },
        };
    },
});
