"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RULE_NAME = void 0;
const node_utils_1 = require("../node-utils");
const create_testing_library_rule_1 = require("../create-testing-library-rule");
exports.RULE_NAME = 'await-fire-event';
exports.default = create_testing_library_rule_1.createTestingLibraryRule({
    name: exports.RULE_NAME,
    meta: {
        type: 'problem',
        docs: {
            description: 'Enforce promises from fire event methods to be handled',
            category: 'Best Practices',
            recommended: false,
        },
        messages: {
            awaitFireEvent: 'Promise returned from `fireEvent.{{ methodName }}` must be handled',
            fireEventWrapper: 'Promise returned from `fireEvent.{{ wrapperName }}` wrapper over fire event method must be handled',
        },
        fixable: null,
        schema: [],
    },
    defaultOptions: [],
    create: function (context, _, helpers) {
        const functionWrappersNames = [];
        function reportUnhandledNode(node, closestCallExpressionNode, messageId = 'awaitFireEvent') {
            if (!node_utils_1.isPromiseHandled(node)) {
                context.report({
                    node: closestCallExpressionNode.callee,
                    messageId,
                    data: { name: node.name },
                });
            }
        }
        function detectFireEventMethodWrapper(node) {
            const innerFunction = node_utils_1.getInnermostReturningFunction(context, node);
            if (innerFunction) {
                functionWrappersNames.push(node_utils_1.getFunctionName(innerFunction));
            }
        }
        return {
            'CallExpression Identifier'(node) {
                if (helpers.isFireEventMethod(node)) {
                    detectFireEventMethodWrapper(node);
                    const closestCallExpression = node_utils_1.findClosestCallExpressionNode(node, true);
                    if (!closestCallExpression) {
                        return;
                    }
                    const references = node_utils_1.getVariableReferences(context, closestCallExpression.parent);
                    if (references.length === 0) {
                        return reportUnhandledNode(node, closestCallExpression);
                    }
                    else {
                        for (const reference of references) {
                            const referenceNode = reference.identifier;
                            return reportUnhandledNode(referenceNode, closestCallExpression);
                        }
                    }
                }
                else if (functionWrappersNames.includes(node.name)) {
                    const closestCallExpression = node_utils_1.findClosestCallExpressionNode(node, true);
                    if (!closestCallExpression) {
                        return;
                    }
                    return reportUnhandledNode(node, closestCallExpression, 'fireEventWrapper');
                }
            },
        };
    },
});
