"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RULE_NAME = void 0;
const experimental_utils_1 = require("@typescript-eslint/experimental-utils");
const utils_1 = require("../utils");
const node_utils_1 = require("../node-utils");
exports.RULE_NAME = 'no-await-sync-events';
const SYNC_EVENTS_REGEXP = new RegExp(`^(${utils_1.SYNC_EVENTS.join('|')})$`);
exports.default = experimental_utils_1.ESLintUtils.RuleCreator(utils_1.getDocsUrl)({
    name: exports.RULE_NAME,
    meta: {
        type: 'problem',
        docs: {
            description: 'Disallow unnecessary `await` for sync events',
            category: 'Best Practices',
            recommended: 'error',
        },
        messages: {
            noAwaitSyncEvents: '`{{ name }}` does not need `await` operator',
        },
        fixable: null,
        schema: [],
    },
    defaultOptions: [],
    create(context) {
        return {
            [`AwaitExpression > CallExpression > MemberExpression > Identifier[name=${SYNC_EVENTS_REGEXP}]`](node) {
                const memberExpression = node.parent;
                const methodNode = memberExpression.property;
                const callExpression = memberExpression.parent;
                const withDelay = callExpression.arguments.length >= 3 &&
                    node_utils_1.isObjectExpression(callExpression.arguments[2]) &&
                    callExpression.arguments[2].properties.some((property) => node_utils_1.isProperty(property) &&
                        experimental_utils_1.ASTUtils.isIdentifier(property.key) &&
                        property.key.name === 'delay');
                if (!(node.name === 'userEvent' &&
                    methodNode.name === 'type' &&
                    withDelay)) {
                    context.report({
                        node: methodNode,
                        messageId: 'noAwaitSyncEvents',
                        data: {
                            name: `${node.name}.${methodNode.name}`,
                        },
                    });
                }
            },
        };
    },
});
