"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RULE_NAME = void 0;
const experimental_utils_1 = require("@typescript-eslint/experimental-utils");
const utils_1 = require("../utils");
const node_utils_1 = require("../node-utils");
exports.RULE_NAME = 'no-multiple-assertions-wait-for';
const WAIT_EXPRESSION_QUERY = 'CallExpression[callee.name=/^(waitFor)$/]';
exports.default = experimental_utils_1.ESLintUtils.RuleCreator(utils_1.getDocsUrl)({
    name: exports.RULE_NAME,
    meta: {
        type: 'suggestion',
        docs: {
            description: "It's preferred to avoid multiple assertions in `waitFor`",
            category: 'Best Practices',
            recommended: false,
        },
        messages: {
            noMultipleAssertionWaitFor: 'Avoid using multiple assertions within `waitFor` callback',
        },
        fixable: null,
        schema: [],
    },
    defaultOptions: [],
    create: function (context) {
        function reportMultipleAssertion(node) {
            const totalExpect = (body) => body.filter((node) => {
                if (node_utils_1.isCallExpression(node.expression) &&
                    node_utils_1.isMemberExpression(node.expression.callee) &&
                    node_utils_1.isCallExpression(node.expression.callee.object)) {
                    const object = node.expression.callee.object;
                    const expressionName = experimental_utils_1.ASTUtils.isIdentifier(object.callee) && object.callee.name;
                    return expressionName === 'expect';
                }
                else {
                    return false;
                }
            });
            if (node_utils_1.isBlockStatement(node) && totalExpect(node.body).length > 1) {
                context.report({
                    node,
                    loc: node.loc.start,
                    messageId: 'noMultipleAssertionWaitFor',
                });
            }
        }
        return {
            [`${WAIT_EXPRESSION_QUERY} > ArrowFunctionExpression > BlockStatement`]: reportMultipleAssertion,
            [`${WAIT_EXPRESSION_QUERY} > FunctionExpression > BlockStatement`]: reportMultipleAssertion,
        };
    },
});
