"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RULE_NAME = void 0;
const experimental_utils_1 = require("@typescript-eslint/experimental-utils");
const utils_1 = require("../utils");
const create_testing_library_rule_1 = require("../create-testing-library-rule");
exports.RULE_NAME = 'no-node-access';
exports.default = create_testing_library_rule_1.createTestingLibraryRule({
    name: exports.RULE_NAME,
    meta: {
        type: 'problem',
        docs: {
            description: 'Disallow direct Node access',
            category: 'Best Practices',
            recommended: 'error',
        },
        messages: {
            noNodeAccess: 'Avoid direct Node access. Prefer using the methods from Testing Library.',
        },
        fixable: null,
        schema: [],
    },
    defaultOptions: [],
    create(context) {
        function showErrorForNodeAccess(node) {
            experimental_utils_1.ASTUtils.isIdentifier(node.property) &&
                utils_1.ALL_RETURNING_NODES.includes(node.property.name) &&
                context.report({
                    node: node,
                    loc: node.property.loc.start,
                    messageId: 'noNodeAccess',
                });
        }
        return {
            ['ExpressionStatement MemberExpression']: showErrorForNodeAccess,
            ['VariableDeclarator MemberExpression']: showErrorForNodeAccess,
        };
    },
});
