"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RULE_NAME = void 0;
const experimental_utils_1 = require("@typescript-eslint/experimental-utils");
const utils_1 = require("../utils");
const node_utils_1 = require("../node-utils");
exports.RULE_NAME = 'no-side-effects-wait-for';
const WAIT_EXPRESSION_QUERY = 'CallExpression[callee.name=/^(waitFor)$/]';
const SIDE_EFFECTS = ['fireEvent', 'userEvent'];
exports.default = experimental_utils_1.ESLintUtils.RuleCreator(utils_1.getDocsUrl)({
    name: exports.RULE_NAME,
    meta: {
        type: 'suggestion',
        docs: {
            description: "It's preferred to avoid side effects in `waitFor`",
            category: 'Best Practices',
            recommended: false,
        },
        messages: {
            noSideEffectsWaitFor: 'Avoid using side effects within `waitFor` callback',
        },
        fixable: null,
        schema: [],
    },
    defaultOptions: [],
    create: function (context) {
        let isImportingTestingLibrary = false;
        function reportSideEffects(node) {
            const hasSideEffects = (body) => body.some((node) => {
                if (node_utils_1.isCallExpression(node.expression) &&
                    node_utils_1.isMemberExpression(node.expression.callee) &&
                    experimental_utils_1.ASTUtils.isIdentifier(node.expression.callee.object)) {
                    const object = node.expression.callee.object;
                    const identifierName = object.name;
                    return SIDE_EFFECTS.includes(identifierName);
                }
                else {
                    return false;
                }
            });
            if (isImportingTestingLibrary &&
                node_utils_1.isBlockStatement(node) &&
                hasSideEffects(node.body)) {
                context.report({
                    node,
                    loc: node.loc.start,
                    messageId: 'noSideEffectsWaitFor',
                });
            }
        }
        return {
            [`${WAIT_EXPRESSION_QUERY} > ArrowFunctionExpression > BlockStatement`]: reportSideEffects,
            [`${WAIT_EXPRESSION_QUERY} > FunctionExpression > BlockStatement`]: reportSideEffects,
            ImportDeclaration(node) {
                isImportingTestingLibrary = utils_1.hasTestingLibraryImportModule(node);
            },
        };
    },
});
