"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RULE_NAME = void 0;
const experimental_utils_1 = require("@typescript-eslint/experimental-utils");
const node_utils_1 = require("../node-utils");
const create_testing_library_rule_1 = require("../create-testing-library-rule");
exports.RULE_NAME = 'no-wait-for-empty-callback';
exports.default = create_testing_library_rule_1.createTestingLibraryRule({
    name: exports.RULE_NAME,
    meta: {
        type: 'suggestion',
        docs: {
            description: "It's preferred to avoid empty callbacks in `waitFor` and `waitForElementToBeRemoved`",
            category: 'Best Practices',
            recommended: 'error',
        },
        messages: {
            noWaitForEmptyCallback: 'Avoid passing empty callback to `{{ methodName }}`. Insert an assertion instead.',
        },
        fixable: null,
        schema: [],
    },
    defaultOptions: [],
    create(context, _, helpers) {
        function isValidWaitFor(node) {
            const parentCallExpression = node.parent;
            const parentIdentifier = node_utils_1.getPropertyIdentifierNode(parentCallExpression);
            return helpers.isAsyncUtil(parentIdentifier, [
                'waitFor',
                'waitForElementToBeRemoved',
            ]);
        }
        function reportIfEmpty(node) {
            if (!isValidWaitFor(node)) {
                return;
            }
            if (node_utils_1.isBlockStatement(node.body) &&
                node.body.body.length === 0 &&
                node_utils_1.isCallExpression(node.parent) &&
                experimental_utils_1.ASTUtils.isIdentifier(node.parent.callee)) {
                context.report({
                    node,
                    loc: node.body.loc.start,
                    messageId: 'noWaitForEmptyCallback',
                    data: {
                        methodName: node.parent.callee.name,
                    },
                });
            }
        }
        function reportNoop(node) {
            if (!isValidWaitFor(node)) {
                return;
            }
            context.report({
                node,
                loc: node.loc.start,
                messageId: 'noWaitForEmptyCallback',
                data: {
                    methodName: node_utils_1.isCallExpression(node.parent) &&
                        experimental_utils_1.ASTUtils.isIdentifier(node.parent.callee) &&
                        node.parent.callee.name,
                },
            });
        }
        return {
            'CallExpression > ArrowFunctionExpression': reportIfEmpty,
            'CallExpression > FunctionExpression': reportIfEmpty,
            'CallExpression > Identifier[name="noop"]': reportNoop,
        };
    },
});
