"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RULE_NAME = void 0;
const node_utils_1 = require("../node-utils");
const create_testing_library_rule_1 = require("../create-testing-library-rule");
exports.RULE_NAME = 'prefer-presence-queries';
exports.default = create_testing_library_rule_1.createTestingLibraryRule({
    name: exports.RULE_NAME,
    meta: {
        docs: {
            category: 'Best Practices',
            description: 'Ensure appropriate get*/query* queries are used with their respective matchers',
            recommended: 'error',
        },
        messages: {
            wrongPresenceQuery: 'Use `getBy*` queries rather than `queryBy*` for checking element is present',
            wrongAbsenceQuery: 'Use `queryBy*` queries rather than `getBy*` for checking element is NOT present',
        },
        schema: [],
        type: 'suggestion',
        fixable: null,
    },
    defaultOptions: [],
    create(context, _, helpers) {
        return {
            'CallExpression Identifier'(node) {
                const expectCallNode = node_utils_1.findClosestCallNode(node, 'expect');
                if (!expectCallNode || !node_utils_1.isMemberExpression(expectCallNode.parent)) {
                    return;
                }
                if (!helpers.isSyncQuery(node)) {
                    return;
                }
                const isPresenceQuery = helpers.isGetQueryVariant(node);
                const expectStatement = expectCallNode.parent;
                const isPresenceAssert = helpers.isPresenceAssert(expectStatement);
                const isAbsenceAssert = helpers.isAbsenceAssert(expectStatement);
                if (!isPresenceAssert && !isAbsenceAssert) {
                    return;
                }
                if (isPresenceAssert && !isPresenceQuery) {
                    return context.report({ node, messageId: 'wrongPresenceQuery' });
                }
                if (isAbsenceAssert && isPresenceQuery) {
                    return context.report({ node, messageId: 'wrongAbsenceQuery' });
                }
            },
        };
    },
});
